@@ -452,8 +452,8 @@ connection_handler (GIOChannel *source, GIOCondition condition, gpointer data)
 	}
 
 	default:
-		g_assert_not_reached ();
-		break;
+		g_critical ("%s: Uncaught case (%d)", G_STRLOC, command);
+		goto comm_fail;
 	}
 
 	return TRUE;
@@ -563,13 +563,13 @@ mail_stub_read_args (MailStub *stub, ...)
 			}
 			if (status == -1)
 				g_array_free (*arr, TRUE);
-			
+
 			break;
 		}
-		
-		
+
+
 		default:
-			g_assert_not_reached ();
+			g_critical ("%s: Uncaught case (%d)", G_STRLOC, argtype);
 			status = -1;
 			break;
 		}
@@ -672,11 +672,11 @@ mail_stub_return_data (MailStub *stub, CamelStubRetval retval, ...)
 				camel_stub_marshal_encode_uint32 (marshal, g_array_index (arr, guint32, i));
 			break;
 		}
-		
-		
+
+
 		default:
-			g_assert_not_reached ();
-			break;
+			g_critical ("%s: Uncaught case (%d)", G_STRLOC, argtype);
+			return;
 		}
 	}
 }
